"""Tests the class that performs calculations on samples"""
from datetime import datetime, timedelta

from cg.meta.report.sample_calculator import SampleCalculator


def test_calculate_processing_days_invalid_delivered(sample_store, helpers):
    """Test calculate processing time when delivered_at is invalid"""

    # GIVEN sample with delivered_at None
    sample = helpers.add_sample(sample_store)
    yesterday = datetime.now() - timedelta(days=1)
    sample.received_at = yesterday
    sample.delivered_at = None

    # WHEN calculate_processing_days
    processing_days = SampleCalculator.calculate_processing_days(sample)

    # THEN calculated processing days is None
    assert processing_days is None


def test_calculate_processing_days_invalid_received(sample_store, helpers):
    """Test calculate processing time when received_at is invalid"""

    # GIVEN sample with no received_at but with delivered_at
    sample = helpers.add_sample(sample_store)
    today = datetime.now()
    sample.received_at = None
    sample.delivered_at = today

    # WHEN calculate_processing_days
    processing_days = SampleCalculator.calculate_processing_days(sample)

    # THEN calculated processing days is None
    assert processing_days is None


def test_calculate_processing_days_valid_dates(sample_store, helpers):
    """Test calculate processing time when all dates are present"""

    # GIVEN sample with received_at and delivered_at
    sample = helpers.add_sample(sample_store)
    yesterday = datetime.now() - timedelta(days=1)
    today = datetime.now()
    sample.received_at = yesterday
    sample.delivered_at = today

    # WHEN calculate_processing_days
    processing_days = SampleCalculator.calculate_processing_days(sample)

    # THEN calculated processing days is 1
    assert processing_days == 1



"""Code for uploading to scout via CLI"""
import logging
from pathlib import Path
from typing import Optional

import click
import yaml
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.scout.scoutapi import ScoutAPI
from cg.meta.upload.scout.scoutapi import UploadScoutAPI
from cg.models.cg_config import CGConfig
from cg.models.scout.scout_load_config import ScoutLoadConfig
from cg.store import Store
from cg.store.models import Family
from housekeeper.store import models as hk_models

from .utils import suggest_cases_to_upload

LOG = logging.getLogger(__name__)


@click.command()
@click.option(
    "-r",
    "--re-upload",
    is_flag=True,
    help="re-upload existing analysis",
)
@click.option("-p", "--print", "print_console", is_flag=True, help="print config values")
@click.argument("case_id", required=False)
@click.pass_context
def scout(context, re_upload: bool, print_console: bool, case_id: str):
    """Upload variants from analysis to Scout."""
    status_db: Store = context.obj.status_db

    LOG.info("----------------- SCOUT -----------------------")

    if not case_id:
        suggest_cases_to_upload(status_db=status_db)
        return

    context.invoke(
        create_scout_load_config, case_id=case_id, print_console=print_console, re_upload=re_upload
    )
    if not print_console:
        context.invoke(upload_case_to_scout, case_id=case_id, re_upload=re_upload)


@click.command(name="create-scout-load-config")
@click.argument("case-id")
@click.option("-p", "--print", "print_console", is_flag=True, help="Only print config")
@click.option("-r", "--re-upload", is_flag=True, help="Delete existing load configs")
@click.pass_obj
def create_scout_load_config(context: CGConfig, case_id: str, print_console: bool, re_upload: bool):
    """Create a load config for a case in scout and add it to housekeeper"""

    scout_upload_api: UploadScoutAPI = context.meta_apis["scout_upload_api"]
    status_db: Store = context.status_db

    LOG.info("----------------- CREATE CONFIG -----------------------")

    LOG.info("Fetching family object")
    case_obj: Family = status_db.family(case_id)
    LOG.info("Create load config")
    if not case_obj.analyses:
        LOG.warning("Could not find analyses for %s", case_id)
        raise click.Abort
    try:
        scout_load_config: ScoutLoadConfig = scout_upload_api.generate_config(case_obj.analyses[0])
    except SyntaxError as err:
        LOG.warning("%s", err)
        raise click.Abort
    LOG.info("Found load config %s", scout_load_config)
    if scout_load_config.track == "cancer":
        root_dir: Path = Path(context.balsamic.root)
    else:
        root_dir: Path = Path(context.mip_rd_dna.root)
    LOG.info("Set root dir to %s", root_dir)
    file_path: Path = root_dir / case_id / "scout_load.yaml"

    if print_console:
        click.echo(yaml.dump(scout_load_config.dict(exclude_none=True)))
        LOG.info("Would save file to %s", file_path)
        return

    if file_path.exists():
        LOG.warning("Scout load config %s already exists", file_path)
        if re_upload:
            LOG.info("Deleting old load config")
            file_path.unlink()
        else:
            LOG.info(
                "You might remove the file and try again, consider that you might also have it in housekeeper"
            )
            raise click.Abort
    LOG.info("Saving config file to disc")
    scout_upload_api.save_config_file(upload_config=scout_load_config, file_path=file_path)

    try:
        LOG.info("Add config file to housekeeper")
        scout_upload_api.add_scout_config_to_hk(
            config_file_path=file_path, case_id=case_id, delete=re_upload
        )
    except FileExistsError as err:
        LOG.warning("%s, consider removing the file from housekeeper and try again", str(err))
        raise click.Abort


@click.command(name="upload-case-to-scout")
@click.option(
    "-r",
    "--re-upload",
    is_flag=True,
    help="re-upload existing analysis",
)
@click.option("--dry-run", is_flag=True)
@click.argument("case_id")
@click.pass_obj
def upload_case_to_scout(context: CGConfig, re_upload: bool, dry_run: bool, case_id: str):
    """Upload variants and case from analysis to Scout."""

    LOG.info("----------------- UPLOAD -----------------------")

    housekeeper_api: HousekeeperAPI = context.housekeeper_api
    scout_api: ScoutAPI = context.scout_api

    tag_name = UploadScoutAPI.get_load_config_tag()
    version_obj = housekeeper_api.last_version(case_id)
    scout_config_file: Optional[hk_models.File] = housekeeper_api.fetch_file_from_version(
        version_obj=version_obj, tags={tag_name}
    )

    if scout_config_file is None:
        raise FileNotFoundError(f"No scout load config was found in housekeeper for {case_id}")

    LOG.info("uploading case %s to scout", case_id)

    if not dry_run:
        scout_api.upload(scout_load_config=scout_config_file.full_path, force=re_upload)

    LOG.info("uploaded to scout using load config %s", scout_config_file.full_path)
    LOG.info("Case loaded successfully to Scout")



"""Test the methods that generate a scout load config"""

import pytest
from cg.constants import Pipeline
from cg.meta.upload.scout.mip_config_builder import MipConfigBuilder
from cg.meta.upload.scout.scoutapi import UploadScoutAPI
from cg.models.scout.scout_load_config import (
    BalsamicLoadConfig,
    MipLoadConfig,
    ScoutLoadConfig,
    ScoutMipIndividual,
)
from cg.store import Store, models

RESULT_KEYS = [
    "family",
    "human_genome_build",
    "rank_model_version",
    "sv_rank_model_version",
]

SAMPLE_FILE_PATHS = ["alignment_path", "chromograph", "vcf2cytosure"]


def test_add_mandatory_info_to_mip_config(
    analysis_obj: models.Analysis, mip_config_builder: MipConfigBuilder
):
    # GIVEN an cg analysis object
    # GIVEN a mip load config object
    assert mip_config_builder.load_config.owner is None
    # GIVEN a file handler with some housekeeper version data

    # WHEN adding the mandatory information
    mip_config_builder.add_mandatory_info_to_load_config()

    # THEN assert mandatory field owner was set
    assert mip_config_builder.load_config.owner


def test_generate_balsamic_load_config(
    balsamic_analysis_obj: models.Analysis, upload_balsamic_analysis_scout_api: UploadScoutAPI
):
    # GIVEN a analysis object that have been run with balsamic
    assert balsamic_analysis_obj.pipeline == Pipeline.BALSAMIC

    # GIVEN a upload scout api with some balsamic information

    # WHEN generating a load config
    config = upload_balsamic_analysis_scout_api.generate_config(analysis_obj=balsamic_analysis_obj)

    # THEN assert that the config is a balsamic config
    assert isinstance(config, BalsamicLoadConfig)


@pytest.mark.parametrize("result_key", RESULT_KEYS)
def test_generate_config_adds_meta_result_key(
    result_key: str,
    mip_analysis_obj: models.Analysis,
    upload_mip_analysis_scout_api: UploadScoutAPI,
):
    """Test that generate config adds the expected result keys"""
    # GIVEN a status db and hk with an analysis
    assert mip_analysis_obj

    # WHEN generating the scout config for the analysis
    result_data: ScoutLoadConfig = upload_mip_analysis_scout_api.generate_config(
        analysis_obj=mip_analysis_obj
    )

    # THEN the config should contain the rank model version used
    assert result_data.dict()[result_key]


def test_generate_config_adds_sample_paths(
    sample_id: str, mip_analysis_obj: models.Analysis, upload_mip_analysis_scout_api: UploadScoutAPI
):
    """Test that generate config adds vcf2cytosure file"""
    # GIVEN a status db and hk with an analysis

    # WHEN generating the scout config for the analysis
    result_data: ScoutLoadConfig = upload_mip_analysis_scout_api.generate_config(mip_analysis_obj)

    # THEN the config should contain the sample file path for each sample
    sample: ScoutMipIndividual
    for sample in result_data.samples:
        if sample.sample_id == sample_id:
            assert sample.vcf2cytosure


def test_generate_config_adds_case_paths(
    sample_id: str, mip_analysis_obj: Store.Analysis, upload_mip_analysis_scout_api: UploadScoutAPI
):
    """Test that generate config adds case file paths"""
    # GIVEN a status db and hk with an analysis

    # WHEN generating the scout config for the analysis
    result_data: ScoutLoadConfig = upload_mip_analysis_scout_api.generate_config(mip_analysis_obj)

    # THEN the config should contain the multiqc file path
    assert result_data.multiqc



"""Tests for the file data class"""
import gzip

import pytest

from cg.models.file_data import FileData


def test_is_gzipped(filled_gzip_file):
    """Test if a file that is gzipped is gzipped"""
    # GIVEN an existing gzipped file
    assert filled_gzip_file.exists()
    # GIVEN a gzipped file with content
    assert filled_gzip_file.stat().st_size > 0

    # WHEN checking if the file is gzipped
    res = FileData.is_gzipped(filled_gzip_file)

    # THEN assert the result is True
    assert res is True


def test_is_gzipped_non_zipped(filled_file):
    """Test if a file that is not gzipped is gzipped"""
    # GIVEN an existing file that is not compressed
    assert filled_file.exists()
    # GIVEN a gzipped file with content
    assert filled_file.stat().st_size > 0

    # WHEN checking if the file is gzipped
    res = FileData.is_gzipped(filled_file)

    # THEN assert the result is False
    assert res is False


def test_is_gzipped_non_existing(non_existing_gzipped_file_path):
    """Test if a file that does not exist is gzipped"""
    # GIVEN an non existing file
    assert not non_existing_gzipped_file_path.exists()

    # WHEN checking if the file is gzipped
    with pytest.raises(FileNotFoundError):
        # THEN assert that a FileNotFoundError is raised
        FileData.is_gzipped(non_existing_gzipped_file_path)


def test_is_gzipped_empty_gzipped_file(gzipped_empty_file):
    """Test if a gzipped file that does exists but is empty"""
    # GIVEN an existing file
    gzipped_file_path = gzipped_empty_file
    assert gzipped_file_path.exists()
    # GIVEN a gzipped file has content (metadata)
    assert gzipped_file_path.stat().st_size > 0

    # WHEN checking if the file is gzipped
    res = FileData.is_gzipped(gzipped_file_path)

    # THEN assert result is since the gzip metadata is in the file
    assert res is True


def test_is_empty_file(gzipped_empty_file):
    """Test if a file that is empty is evaluated as empty"""
    # GIVEN an existing file
    gzipped_file_path = gzipped_empty_file
    assert gzipped_file_path.exists()
    # GIVEN a gzipped file without some metadata content
    assert gzipped_file_path.stat().st_size > 0
    # GIVEN that there is no non-metadata content in the file
    with gzip.open(gzipped_file_path, "r") as infile:
        content = infile.read()
    assert content == b""

    # WHEN checking if the file is empty
    res = FileData.is_empty(gzipped_file_path)

    # THEN assert result is True
    assert res is True


def test_is_empty_file_not_empty(filled_gzip_file):
    """Test if a file that is not empty is evaluated as empty"""
    # GIVEN a gzipped file with content
    assert filled_gzip_file.stat().st_size > 0

    # WHEN checking if the file is empty
    res = FileData.is_empty(filled_gzip_file)

    # THEN assert result is False
    assert res is False



import logging

from cg.models.cg_config import CGConfig
from cg.store import Store


def test_instantiate_correct_configs(base_config_dict: dict):
    # GIVEN a dictionary with the basic configs

    # WHEN instantiating a CGConfig object
    config_object = CGConfig(**base_config_dict)

    # THEN assert that it was successfully created
    assert isinstance(config_object, CGConfig)


def test_fetching_the_status_db(base_config_dict: dict, caplog):
    caplog.set_level(logging.DEBUG)
    # GIVEN a dictionary with the basic configs

    # WHEN instantiating a CGConfig object
    config_object = CGConfig(**base_config_dict)

    # THEN assert that the status db exists
    assert isinstance(config_object.status_db, Store)
    # THEN assert that it was communicated that it was instantiated
    assert "Instantiating status db" in caplog.text


def test_api_instantiated_once(base_config_dict: dict, caplog):
    caplog.set_level(logging.DEBUG)
    # GIVEN a dictionary with the basic configs
    # GIVEN a CGConfig object
    config_object = CGConfig(**base_config_dict)

    # WHEN fetching (instantiating) the status_db

    # THEN assert that it was only instantiated once
    config_object.status_db
    assert "Instantiating status db" in caplog.text
    caplog.clear()
    config_object.status_db
    assert "Instantiating status db" not in caplog.text



"""Constants specific for compression"""
import datetime

# Constants for crunchy
FASTQ_FIRST_READ_SUFFIX = "_R1_001.fastq.gz"
FASTQ_SECOND_READ_SUFFIX = "_R2_001.fastq.gz"
SPRING_SUFFIX = ".spring"

# Number of days until fastqs counts as old
FASTQ_DELTA = 21
# Get the fastq delta in datetime format
FASTQ_DATETIME_DELTA = datetime.timedelta(days=FASTQ_DELTA)

PROBLEMATIC_CASES = [
    "causalmite",
    "deepcub",
    "expertalien",
    "fluenteagle",
    "grandkoi",
    "lovingmayfly",
    "loyalegret",
    "modernbee",
    "proudcollie",
    "richalien",
    "suremako",
    "wisestork",
]

# List of cases used for validation that we should skip
BALSAMIC_VALIDATION_CASES = [
    "bosssponge",  # BALSAMIC validation case tumor-only panel
    "civilsole",  # BALSAMIC validation case tumor-only wgs
    "fleetjay",  # BALSAMIC validation case tumor-normal wgs
    "moralgoat",  # BALSAMIC validation case tumor-normal wgs
    "sweetelf",  # BALSAMIC positive control tumor-only panel
    "unitedbeagle",  # BALSAMIC validation case tumor-normal panel
]

MIP_VALIDATION_CASES = [
    "brightcaiman",  # DNA rare disease positive control
    "casualgannet",  # DNA rare disease positive control
    "civilkoala",  # RNA rare disease positive control
    "cleanshrimp",  # DNA rare disease positive control
    "drivenmolly",  # RNA rare disease positive control
    "easybeetle",  # DNA rare disease positive control
    "epicasp",  # DNA rare disease positive control
    "expertmole",  # RNA rare disease positive control
    "finequagga",  # RNA rare disease positive control
    "firstfawn",  # DNA rare disease positive control
    "gladthrush",  # DNA rare disease positive control
    "helpedfilly",  # DNA rare disease positive control
    "hotskink",  # DNA rare disease positive control
    "inferret",  # DNA rare disease positive control
    "intentcorgi",  # DNA rare disease positive control
    "intentmayfly",  # DNA rare disease positive control
    "justhusky",  # DNA rare disease positive control
    "kindcaiman",  # DNA rare disease positive control
    "lightprawn",  # DNA rare disease positive control
    "livingox",  # DNA rare disease positive control
    "newaphid",  # RNA rare disease positive control
    "nextjackal",  # DNA rare disease positive control
    "modernmule",  # DNA rare disease positive control
    "moralcattle",  # RNA rare disease positive control
    "onemite",  # DNA rare disease positive control
    "proudcougar",  # DNA rare disease positive control
    "rightmacaw",  # DNA rare disease positive control
    "safeguinea",  # DNA rare disease positive control
    "sharpparrot",  # RNA rare disease positive control
    "sharppigeon",  # DNA rare disease positive control
    "sharpwhale",  # DNA rare disease positive control
    "smoothboa",  # RNA rare disease positive control
    "strongbison",  # DNA rare disease positive control
    "tenderoriole",  # DNA rare disease positive control
    "topsrhino",  # DNA rare disease positive control
    "usablemarten",  # DNA rare disease positive control
    "vitalmouse",  # DNA rare disease positive control
]

OTHER_VALIDATION_CASES = [
    "bigdrum",
    "busycolt",
    "daringpony",
    "frankhusky",
    "gamedeer",
    "hotviper",
    "keencalf",
    "keenviper",
    "luckyhog",
    "meetpossum",
    "mintbaboon",
    "mintyeti",
    "proeagle",
    "propercoral",
    "pumpedcat",
    "strongman",
]

CASES_TO_IGNORE = (
    PROBLEMATIC_CASES + OTHER_VALIDATION_CASES + BALSAMIC_VALIDATION_CASES + MIP_VALIDATION_CASES
)



from tests.cli.demultiplex.conftest import (
    fixture_demultiplex_configs,
    fixture_demultiplex_context,
    fixture_demultiplex_fixtures,
    fixture_demultiplexed_flowcell_working_directory,
    fixture_demultiplexed_flowcells_working_directory,
    fixture_demultiplexing_api,
    fixture_demux_results_not_finished_dir,
    fixture_demux_run_dir,
    fixture_flowcell_full_name,
    fixture_flowcell_object,
    fixture_flowcell_runs_working_directory,
    fixture_sbatch_process,
    fixture_stats_api,
)



"""Fixtures for cli clean tests"""

import datetime
from pathlib import Path

import pytest
from cg.constants import Pipeline
from cg.meta.workflow.balsamic import BalsamicAnalysisAPI
from cg.models.cg_config import CGConfig
from tests.store_helpers import StoreHelpers


@pytest.fixture()
def clean_context(
    cg_context: CGConfig,
    helpers: StoreHelpers,
    project_dir: Path,
    timestamp_yesterday: datetime.datetime,
    timestamp_today: datetime.datetime,
) -> CGConfig:
    analysis_api = BalsamicAnalysisAPI(cg_context)
    store = analysis_api.status_db

    # Create textbook case for cleaning
    case_to_clean = helpers.add_case(
        store=store, internal_id="balsamic_case_clean", case_id="balsamic_case_clean"
    )
    sample_case_to_clean = helpers.add_sample(
        store,
        internal_id="balsamic_sample_clean",
        is_tumour=True,
        application_type="wgs",
        data_analysis=Pipeline.BALSAMIC,
    )
    helpers.add_relationship(store, case=case_to_clean, sample=sample_case_to_clean)

    helpers.add_analysis(
        store,
        case=case_to_clean,
        pipeline=Pipeline.BALSAMIC,
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )
    Path(analysis_api.get_case_path("balsamic_case_clean")).mkdir(exist_ok=True, parents=True)

    # Create textbook case not for cleaning
    case_to_not_clean = helpers.add_case(
        store=store, internal_id="balsamic_case_not_clean", case_id="balsamic_case_not_clean"
    )
    case_to_not_clean.action = "running"
    store.commit()

    sample_case_to_not_clean = helpers.add_sample(
        store,
        internal_id="balsamic_sample_not_clean",
        is_tumour=True,
        application_type="wgs",
        data_analysis="balsamic",
    )
    helpers.add_relationship(store, case=case_to_not_clean, sample=sample_case_to_not_clean)

    helpers.add_analysis(
        store,
        case=case_to_not_clean,
        pipeline="balsamic",
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )
    Path(analysis_api.get_case_path("balsamic_case_not_clean")).mkdir(exist_ok=True, parents=True)
    cg_context.meta_apis["analysis_api"] = analysis_api

    cg_context.data_delivery.base_path = f"{project_dir}/rsync"

    return cg_context


@pytest.fixture(name="rsync_process")
def fixture_rsync_process(project_dir: Path) -> Path:
    """Return a rsync process after ensuing that is is created"""

    rsync_process = project_dir / "rsync" / "rsync_process"

    rsync_process.mkdir(exist_ok=True, parents=True)
    return rsync_process



"""Code for uploading genotype data via CLI"""
import logging

import click

from cg.exc import CgError
from cg.meta.upload.gisaid import GisaidAPI
from cg.models.cg_config import CGConfig
from cg.models.email import EmailInfo
from cg.utils.email import send_mail

LOG = logging.getLogger(__name__)


@click.command()
@click.argument("case_id", required=True)
@click.pass_obj
def gisaid(context: CGConfig, case_id: str):
    """Upload mutant analysis data to GISAID."""

    LOG.info("----------------- GISAID UPLOAD -------------------")

    gisaid_api = GisaidAPI(config=context)

    try:
        gisaid_api.upload(case_id=case_id)
        LOG.info("Upload to GISAID successful")
    except CgError as error:
        email_info: EmailInfo = EmailInfo(
            **gisaid_api.email_base_settings.dict(),
            receiver_email=gisaid_api.log_watch,
            message=error.message,
            subject="Gisaid Upload Failed",
        )
        send_mail(email_info)
        LOG.error("Upload Failed")
        raise



import datetime as dt
from typing import List

import pytest
from cg.constants import DataDelivery

from cg.exc import OrderError
from cg.meta.orders import OrdersAPI, OrderType
from cg.models.orders.order import OrderIn
from cg.store import models, Store
from cgmodels.cg.constants import Pipeline
import cg

PROCESS_LIMS_FUNCTION = "cg.meta.orders.api.process_lims"


@pytest.mark.parametrize(
    "order_type",
    [
        OrderType.BALSAMIC,
        OrderType.EXTERNAL,
        OrderType.FASTQ,
        OrderType.METAGENOME,
        OrderType.MICROSALT,
        OrderType.MIP_DNA,
        OrderType.MIP_RNA,
        OrderType.RML,
        OrderType.SARS_COV_2,
    ],
)
def test_submit(
    base_store,
    orders_api: OrdersAPI,
    all_orders_to_submit,
    monkeypatch,
    order_type,
    user_name: str,
    user_mail: str,
    ticket_number: int,
):
    order_data = all_orders_to_submit[order_type]
    lims_project_data = {"id": "ADM1234", "date": dt.datetime.now()}
    lims_map = {
        sample["name"]: f"ELH123A{index}" for index, sample in enumerate(order_data.samples)
    }
    monkeypatch.setattr(
        PROCESS_LIMS_FUNCTION,
        lambda **kwargs: (lims_project_data, lims_map),
    )

    # GIVEN an order and an empty store
    assert base_store.samples().first() is None

    # WHEN submitting the order

    result = orders_api.submit(
        project=order_type, order_in=order_data, user_name=user_name, user_mail=user_mail
    )

    # THEN the result should contain the ticket number for the order
    for record in result["records"]:
        if isinstance(record, (models.Pool, models.Sample)):
            assert record.ticket_number == ticket_number
        elif isinstance(record, models.Family):
            for link_obj in record.links:
                assert link_obj.sample.ticket_number == ticket_number


@pytest.mark.parametrize(
    "order_type",
    [OrderType.MIP_DNA, OrderType.MIP_RNA, OrderType.EXTERNAL, OrderType.BALSAMIC],
)
def test_submit_illegal_sample_customer(
    sample_store,
    orders_api,
    all_orders_to_submit,
    monkeypatch,
    order_type,
    ticket_number: int,
    user_name: str,
    user_mail: str,
):

    order_data = all_orders_to_submit[order_type]
    lims_project_data = {"id": "ADM1234", "date": dt.datetime.now()}
    lims_map = {
        sample["name"]: f"ELH123A{index}" for index, sample in enumerate(order_data.samples)
    }
    monkeypatch.setattr(PROCESS_LIMS_FUNCTION, lambda **kwargs: (lims_project_data, lims_map))

    # GIVEN we have an order with a customer that is not in the same customer group as customer
    # that the samples originate from
    customer_group = sample_store.add_customer_group("customer999only", "customer 999 only group")
    sample_store.add_commit(customer_group)
    new_customer = sample_store.add_customer(
        "customer999",
        "customer 999",
        scout_access=True,
        invoice_address="dummy street",
        customer_group=customer_group,
        invoice_reference="dummy nr",
    )
    sample_store.add_commit(new_customer)
    existing_sample = sample_store.samples().first()
    existing_sample.customer = new_customer
    sample_store.add_commit(existing_sample)

    for sample in order_data.samples:
        sample["internal_id"] = existing_sample.internal_id

    # WHEN calling submit
    # THEN an OrderError should be raised on illegal customer
    with pytest.raises(OrderError):
        orders_api.submit(
            project=order_type, order_in=order_data, user_name=user_name, user_mail=user_mail
        )


@pytest.mark.parametrize(
    "order_type",
    [OrderType.MIP_DNA, OrderType.MIP_RNA, OrderType.EXTERNAL, OrderType.BALSAMIC],
)
def test_submit_scout_legal_sample_customer(
    sample_store,
    orders_api,
    all_orders_to_submit,
    monkeypatch,
    order_type,
    user_name: str,
    user_mail: str,
    ticket_number: int,
):

    order_data = all_orders_to_submit[order_type]
    lims_project_data = {"id": "ADM1234", "date": dt.datetime.now()}
    lims_map = {
        sample["name"]: f"ELH123A{index}" for index, sample in enumerate(order_data.samples)
    }
    monkeypatch.setattr(PROCESS_LIMS_FUNCTION, lambda **kwargs: (lims_project_data, lims_map))
    # GIVEN we have an order with a customer that is in the same customer group as customer
    # that the samples originate from
    customer_group = sample_store.add_customer_group("customer999only", "customer 999 only group")
    sample_store.add_commit(customer_group)
    sample_customer = sample_store.add_customer(
        "customer1",
        "customer 1",
        scout_access=True,
        invoice_address="dummy street 1",
        customer_group=customer_group,
        invoice_reference="dummy nr",
    )
    order_customer = sample_store.add_customer(
        "customer2",
        "customer 2",
        scout_access=True,
        invoice_address="dummy street 2",
        customer_group=customer_group,
        invoice_reference="dummy nr",
    )
    sample_store.add_commit(sample_customer)
    sample_store.add_commit(order_customer)
    existing_sample = sample_store.samples().first()
    existing_sample.customer = sample_customer
    sample_store.commit()
    order_data.customer = order_customer.internal_id
    first = True

    for sample in order_data.samples:
        if first:
            sample["internal_id"] = existing_sample.internal_id
            first = False

    # WHEN calling submit
    # THEN an OrderError should not be raised on illegal customer
    orders_api.submit(
        project=order_type, order_in=order_data, user_name=user_name, user_mail=user_mail
    )


@pytest.mark.parametrize(
    "order_type",
    [OrderType.RML, OrderType.FASTQ, OrderType.MICROSALT, OrderType.METAGENOME],
)
def test_submit_non_scout_legal_sample_customer(
    sample_store,
    orders_api,
    all_orders_to_submit,
    monkeypatch,
    order_type,
    user_name: str,
    user_mail: str,
    ticket_number: int,
):

    order_data = all_orders_to_submit[order_type]
    lims_project_data = {"id": "ADM1234", "date": dt.datetime.now()}
    lims_map = {
        sample["name"]: f"ELH123A{index}" for index, sample in enumerate(order_data.samples)
    }
    monkeypatch.setattr(PROCESS_LIMS_FUNCTION, lambda **kwargs: (lims_project_data, lims_map))

    # GIVEN we have an order with a customer that is in the same customer group as customer
    # that the samples originate from but on order types where this is dis-allowed
    customer_group = sample_store.add_customer_group("customer999only", "customer 999 only group")
    sample_store.add_commit(customer_group)
    sample_customer = sample_store.add_customer(
        "customer1",
        "customer 1",
        scout_access=True,
        invoice_address="dummy street 1",
        customer_group=customer_group,
        invoice_reference="dummy nr",
    )
    order_customer = sample_store.add_customer(
        "customer2",
        "customer 2",
        scout_access=True,
        invoice_address="dummy street 2",
        customer_group=customer_group,
        invoice_reference="dummy nr",
    )
    sample_store.add_commit(sample_customer)
    sample_store.add_commit(order_customer)
    existing_sample = sample_store.samples().first()
    existing_sample.customer = sample_customer
    sample_store.commit()
    order_data.customer = order_customer.internal_id
    first = True

    for sample in order_data.samples:
        if first:
            sample["internal_id"] = existing_sample.internal_id
            first = False

    # WHEN calling submit
    # THEN an OrderError should be raised on illegal customer
    with pytest.raises(OrderError):
        orders_api.submit(
            project=order_type, order_in=order_data, user_name=user_name, user_mail=user_mail
        )


def test_submit_duplicate_sample_case_name(
    orders_api, mip_order_to_submit, ticket_number: int, user_name: str, user_mail: str, mocker
):
    # GIVEN we have an order with a case that is already in the database
    order_data = OrderIn.parse_obj(mip_order_to_submit)
    store = orders_api.status
    customer_obj = store.customer(order_data.customer)

    for sample in order_data.samples:
        case_id = sample["family_name"]
        if not store.find_family(customer=customer_obj, name=case_id):
            case_obj = store.add_case(
                data_analysis=Pipeline.MIP_DNA,
                data_delivery=DataDelivery.SCOUT,
                name=case_id,
                panels=None,
            )
            case_obj.customer = customer_obj
            store.add_commit(case_obj)
        assert store.find_family(customer=customer_obj, name=case_id)

    mocker.patch(PROCESS_LIMS_FUNCTION)

    # WHEN calling submit
    # THEN an OrderError should be raised on duplicate case name
    with pytest.raises(OrderError):
        orders_api.submit(
            project=OrderType.MIP_DNA, order_in=order_data, user_name=user_name, user_mail=user_mail
        )

    # Then no new samples should have been created in LIMS
    cg.meta.orders.api.process_lims.assert_not_called()


def test_submit_unique_sample_case_name(
    orders_api, mip_order_to_submit, ticket_number: int, user_name: str, user_mail: str, monkeypatch
):
    # GIVEN we have an order with a case that is not existing in the database
    order_data = OrderIn.parse_obj(mip_order_to_submit)
    store = orders_api.status

    for sample in order_data.samples:
        case_id = sample["family_name"]
        customer_obj = store.customer(order_data.customer)
        assert not store.find_family(customer=customer_obj, name=case_id)

    lims_project_data = {"id": "ADM1234", "date": dt.datetime.now()}
    lims_map = {
        sample["name"]: f"ELH123A{index}" for index, sample in enumerate(order_data.samples)
    }
    monkeypatch.setattr(PROCESS_LIMS_FUNCTION, lambda **kwargs: (lims_project_data, lims_map))

    # WHEN calling submit
    orders_api.submit(
        project=OrderType.MIP_DNA, order_in=order_data, user_name=user_name, user_mail=user_mail
    )

    # Then no exception about duplicate names should be thrown



from pathlib import Path

import pytest

from cg.constants.priority import SlurmQos
from cg.models.mip.mip_config import MipBaseConfig
from cg.models.mip.mip_sample_info import MipBaseSampleInfo


@pytest.fixture(name="mip_case_config_dna")
def fixture_mip_case_config_dna(fixtures_dir) -> Path:
    """Return path to MIP DNA case_config.yaml"""
    return Path(fixtures_dir, "apps", "mip", "dna", "store", "case_config.yaml")


@pytest.fixture(name="mip_analysis_config_dna_raw")
def fixture_mip_analysis_config_dna_raw() -> dict:
    """Raw MIP DNA analysis config"""
    return {
        "analysis_type": {"sample_id": "wgs"},
        "case_id": "yellowhog",
        "config_file_analysis": "a_config_path",
        "email": "someone@somewhere.com",
        "family_id": "a_family_id",
        "dry_run_all": False,
        "log_file": "a_log_path",
        "outdata_dir": "tests/fixtures/apps/mip/dna/store",
        "sample_info_file": "tests/fixtures/apps/mip/dna/store/case_qc_sample_info.yaml",
        "slurm_quality_of_service": SlurmQos.LOW,
        "store_file": "tests/fixtures/apps/mip/dna/store/case_id_deliverables.yaml",
    }


@pytest.fixture(name="mip_analysis_config_dna")
def fixture_mip_analysis_config_dna(mip_analysis_config_dna_raw: dict) -> MipBaseConfig:
    return MipBaseConfig(**mip_analysis_config_dna_raw)


@pytest.fixture(name="sample_info_dna_raw")
def fixture_sample_info_dna_raw() -> dict:
    """Raw sample_info fixture"""
    return {
        "analysisrunstatus": "finished",
        "analysis_date": "2021-05-05T16:16:01",
        "case_id": "yellowhog",
        "family_id": "a_family_id",
        "human_genome_build": {"version": 37, "source": "grch"},
        "mip_version": "v9.0.0",
        "recipe": {
            "genmod": {
                "rank_model": {"version": "v1.0"},
            },
            "sv_genmod": {
                "sv_rank_model": {"version": "v1.2.0"},
            },
        },
    }


@pytest.fixture(name="sample_info_dna")
def fixture_sample_info_dna(sample_info_dna_raw: dict) -> MipBaseSampleInfo:
    return MipBaseSampleInfo(**sample_info_dna_raw)



"""Tests for the compress data class"""
import os
from datetime import datetime
from pathlib import Path

from cg.models import CompressionData


def test_get_run_name():
    """Test that the correct run name is returned"""
    # GIVEN a file path that ends with a run name
    file_path = Path("/path/to/dir")
    run_name = "a_run"
    # GIVEN a compression data object
    compression_obj = CompressionData(file_path / run_name)

    # WHEN fetching the run name
    # THEN assert the correct run name is returned
    assert compression_obj.run_name == run_name


def test_get_change_date(compression_object):
    """Test to get the date time for when a file was changed"""
    # GIVEN a existing file
    file_path = compression_object.spring_path
    file_path.touch()

    # WHEN fetching the date when file was created
    change_date = compression_object.get_change_date(file_path)

    # THEN check that it is the same date as today
    assert change_date.date() == datetime.today().date()


def test_get_nlinks_one_link(compression_object):
    """Test get_nlinks when there is one link to a file"""
    # GIVEN a file with one link
    file_path = compression_object.spring_path
    file_path.touch()

    # WHEN getting number of links to inode
    nlinks = compression_object.get_nlinks(file_path)

    # THEN number of links should be one
    assert nlinks == 1


def test_get_nlinks_two_links(compression_object):
    """Test get_nlinks two links"""
    # GIVEN a file with two links
    file_path = compression_object.spring_path
    file_path.touch()

    first_link = file_path.parent / "link-1"
    os.link(file_path, first_link)

    # WHEN fetching the number of links from the API
    nlinks = compression_object.get_nlinks(file_path)

    # THEN number of links should be two
    assert nlinks == 2


def test_get_nlinks_three_links(compression_object):
    """Test get_nlinks when three links"""
    # GIVEN a file with two links
    file_path = compression_object.spring_path
    file_path.touch()

    first_link = file_path.parent / "link-1"
    os.link(file_path, first_link)

    second_link = file_path.parent / "link-2"
    os.link(file_path, second_link)

    # WHEN fetching the links from the api
    nlinks = compression_object.get_nlinks(file_path)

    # THEN number of links should be three
    assert nlinks == 3



"""Tests for delivery API"""

from pathlib import Path
from typing import List, Set

from tests.store_helpers import StoreHelpers

from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.meta.deliver import DeliverAPI
from cg.store import Store
from cg.store.models import FamilySample, Sample
from housekeeper.store import models as hk_models


def test_get_delivery_path(
    base_store: Store, real_housekeeper_api: HousekeeperAPI, project_dir: Path, case_id: str
):
    """Test to create the delivery path"""
    # GIVEN a deliver api
    deliver_api = DeliverAPI(
        store=base_store,
        hk_api=real_housekeeper_api,
        case_tags=["case-tag"],
        sample_tags=["sample-tag"],
        project_base_path=project_dir,
        delivery_type="balsamic",
    )
    customer_id = "cust000"
    ticket_id = 1234
    deliver_api._set_customer_id(customer_id)
    deliver_api._set_ticket_id(ticket_id)

    # WHEN fetching the deliver path
    deliver_path = deliver_api.create_delivery_dir_path(case_name=case_id)

    # THEN assert that the path looks like expected
    assert deliver_path == project_dir / customer_id / "inbox" / str(ticket_id) / case_id


def test_get_case_analysis_files(populated_deliver_api: DeliverAPI, case_id: str):
    """Test to fetch case specific files for a case that exists in housekeeper"""
    deliver_api: DeliverAPI = populated_deliver_api
    # GIVEN a case which exist as bundle in hk with a version
    version_obj = deliver_api.hk_api.last_version(case_id)
    assert version_obj

    # GIVEN that a case object exists in the database
    link_objs: List[FamilySample] = deliver_api.store.family_samples(case_id)
    samples: List[Sample] = [link.sample for link in link_objs]
    sample_ids: Set[str] = set([sample.internal_id for sample in samples])

    # WHEN fetching all case files from the delivery api
    bundle_latest_files = deliver_api.get_case_files_from_version(
        version_obj=version_obj, sample_ids=sample_ids
    )

    ## THEN housekeeper files should be returned
    assert bundle_latest_files


def test_get_case_files_from_version(
    analysis_store: Store,
    case_id: str,
    real_housekeeper_api: HousekeeperAPI,
    case_hk_bundle_no_files: dict,
    bed_file: str,
    vcf_file: Path,
    project_dir: Path,
    helpers=StoreHelpers,
):
    # GIVEN a store with a case
    case_obj = analysis_store.family(case_id)
    assert case_obj.internal_id == case_id
    # GIVEN a delivery api
    deliver_api = DeliverAPI(
        store=analysis_store,
        hk_api=real_housekeeper_api,
        case_tags=[{"case-tag"}],
        sample_tags=[{"sample-tag"}],
        project_base_path=project_dir,
        delivery_type="balsamic",
    )

    # GIVEN a housekeeper db populated with a bundle including a case specific file and a sample specific file
    case_hk_bundle_no_files["files"] = [
        {"path": bed_file, "archive": False, "tags": ["case-tag"]},
        {"path": str(vcf_file), "archive": False, "tags": ["sample-tag", "ADM1"]},
    ]
    helpers.ensure_hk_bundle(real_housekeeper_api, bundle_data=case_hk_bundle_no_files)

    # GIVEN a version object where two file exists
    version_obj: hk_models.Version = real_housekeeper_api.last_version(case_id)
    assert len(version_obj.files) == 2

    # GIVEN the sample ids of the samples
    link_objs: List[FamilySample] = analysis_store.family_samples(case_id)
    samples: List[Sample] = [link.sample for link in link_objs]
    sample_ids: Set[str] = set([sample.internal_id for sample in samples])

    # WHEN fetching the case files
    case_files = deliver_api.get_case_files_from_version(
        version_obj=version_obj, sample_ids=sample_ids
    )

    # THEN we should only get the case specific files back
    nr_files: int = 0
    case_file: Path
    for nr_files, case_file in enumerate(case_files, 1):
        assert case_file.name == Path(bed_file).name
    # THEN assert that only the case-tag file was returned
    assert nr_files == 1


def test_get_sample_files_from_version(
    deliver_api: DeliverAPI,
    case_hk_bundle_no_files: dict,
    bed_file: str,
    vcf_file: Path,
    project_dir: Path,
    case_id: str,
    helpers=StoreHelpers,
):
    """Test to fetch sample specific files from the deliver API

    The purpose of the test is to see that only sample specific files are returned
    """
    # GIVEN a case which exist as bundle in hk
    # GIVEN a housekeeper db populated with a bundle including a case specific file and a sample specific file
    hk_api = deliver_api.hk_api
    case_hk_bundle_no_files["files"] = [
        {"path": bed_file, "archive": False, "tags": ["case-tag"]},
        {"path": str(vcf_file), "archive": False, "tags": ["sample-tag", "ADM1"]},
    ]
    helpers.ensure_hk_bundle(hk_api, bundle_data=case_hk_bundle_no_files)
    # GIVEN a version object with some files
    version_obj: hk_models.Version = hk_api.last_version(case_id)
    assert len(version_obj.files) == 2

    # WHEN fetching the sample specific files
    sample_files = deliver_api.get_sample_files_from_version(
        version_obj=version_obj, sample_id="ADM1"
    )

    # THEN assert that only the sample specific file was returned
    nr_files: int = 0
    sample_file: Path
    for nr_files, sample_file in enumerate(sample_files, 1):
        assert sample_file.name == vcf_file.name
    # THEN assert that only the sample-tag file was returned
    assert nr_files == 1



"""Maps tag info from housekeeper tags to scout load config"""

from typing import Set

from pydantic import BaseModel, Field


class CaseTags(BaseModel):
    snv_vcf: Set[str] = Field(..., description="vcf_snv for rare disease and vcf_cancer for cancer")
    snv_research_vcf: Set[str] = Field(None, description="vcf_snv_research for rare disease")
    sv_vcf: Set[str] = Field(
        ..., description="vcf_cancer_sv for rare disease and vcf_sv_cancer for cancer"
    )
    sv_research_vcf: Set[str] = Field(None, description="vcf_sv_research for rare disease")
    vcf_str: Set[str] = Field(
        None, description="Short Tandem Repeat variants, only for rare disease"
    )
    smn_tsv: Set[str] = Field(None, description="SMN gene variants, only for rare disease")
    peddy_ped: Set[str] = Field(None, description="Ped info from peddy, only for rare disease")
    peddy_sex: Set[str] = Field(None, description="Peddy sex check, only for rare disease")
    peddy_check: Set[str] = Field(None, description="Peddy pedigree check, only for rare disease")
    multiqc_report: Set[str]
    delivery_report: Set[str] = Field(None, description="Delivery report for cancer")


class SampleTags(BaseModel):
    # If cram does not exist
    bam_file: Set[str]
    alignment_file: Set[str]
    vcf2cytosure: Set[str] = None
    mt_bam: Set[str] = None
    chromograph_autozyg: Set[str] = None
    chromograph_coverage: Set[str] = None
    chromograph_regions: Set[str] = None
    chromograph_sites: Set[str] = None



"""Interactions with the gisaid cli upload_results_to_gisaid"""
import json
import logging
from datetime import date
from pathlib import Path
from typing import List, Dict, Optional

from housekeeper.store.models import File
import tempfile

from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.lims import LimsAPI
from cg.models.cg_config import CGConfig
from cg.store import models, Store
from cg.utils import Process
from .constants import HEADERS
from .models import GisaidSample, UploadFiles, GisaidAccession
import csv

from cg.exc import (
    FastaSequenceMissingError,
    AccessionNumerMissingError,
    GisaidUploadFailedError,
    InvalidFastaError,
    HousekeeperFileMissingError,
)

LOG = logging.getLogger(__name__)


class GisaidAPI:
    """Interface with Gisaid cli uppload"""

    def __init__(self, config: CGConfig):
        self.housekeeper_api: HousekeeperAPI = config.housekeeper_api
        self.lims_api: LimsAPI = config.lims_api
        self.status_db: Store = config.status_db
        self.gisaid_submitter: str = config.gisaid.submitter
        self.gisaid_binary: str = config.gisaid.binary_path
        self.gisaid_log_dir: str = config.gisaid.log_dir
        self.log_watch: str = config.gisaid.logwatch_email
        self.email_base_settings = config.email_base_settings

        self.process = Process(binary=self.gisaid_binary)

    def get_gisaid_samples(self, case_id: str) -> List[GisaidSample]:
        """Get list of Gisaid sample objects."""

        samples: List[models.Sample] = self.status_db.get_sequenced_samples(family_id=case_id)
        gisaid_samples = []
        for sample in samples:
            sample_id: str = sample.internal_id

            gisaid_sample = GisaidSample(
                case_id=case_id,
                cg_lims_id=sample_id,
                covv_subm_sample_id=sample.name,
                submitter=self.gisaid_submitter,
                fn=f"{case_id}.fasta",
                covv_collection_date=str(
                    self.lims_api.get_sample_attribute(lims_id=sample_id, key="collection_date")
                ),
                region=self.lims_api.get_sample_attribute(lims_id=sample_id, key="region"),
                region_code=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="region_code"
                ),
                covv_orig_lab=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="original_lab"
                ),
                covv_orig_lab_addr=self.lims_api.get_sample_attribute(
                    lims_id=sample_id, key="original_lab_address"
                ),
            )
            gisaid_samples.append(gisaid_sample)
        return gisaid_samples

    def get_gisaid_fasta(self, gisaid_samples: List[GisaidSample], case_id: str) -> List[str]:
        """Fetch a fasta files form house keeper for batch upload_results_to_gisaid to gisaid"""
        fasta_file: File = self.housekeeper_api.find_file_in_latest_version(
            case_id=case_id, tags=["consensus"]
        )

        gisaid_delivery_fasta = []
        with open(str(fasta_file.full_path)) as handle:
            fasta_lines = handle.readlines()
            for line in fasta_lines:
                if line[0] == ">":
                    gisaid_delivery_fasta.append(
                        self.get_new_header(old_header=line, samples=gisaid_samples)
                    )
                else:
                    gisaid_delivery_fasta.append(line)
        if not gisaid_delivery_fasta:
            raise InvalidFastaError(message="Empty fasta file")
        return gisaid_delivery_fasta

    def get_new_header(self, samples: List[GisaidSample], old_header: str) -> str:
        for sample in samples:
            sample_id = sample.covv_subm_sample_id
            if old_header.find(sample_id) != -1:
                return f">{sample.covv_virus_name}\n"
        raise FastaSequenceMissingError

    def build_gisaid_fasta(
        self, gisaid_samples: List[GisaidSample], file_name: str, case_id: str
    ) -> Path:
        """Writing a new fasta with headers adjusted for gisaid upload_results_to_gisaid"""

        file: Path = Path(file_name)
        fasta_lines: List[str] = self.get_gisaid_fasta(
            gisaid_samples=gisaid_samples, case_id=case_id
        )

        with open(file, "w") as write_file_obj:
            write_file_obj.writelines(fasta_lines)
        return file

    def get_sample_row(self, gisaid_sample: GisaidSample) -> List[str]:
        """Build row for a sample in the batch upload_results_to_gisaid csv."""

        sample_dict = gisaid_sample.dict()
        return [sample_dict.get(header, "") for header in HEADERS]

    def get_sample_rows(self, gisaid_samples: List[GisaidSample]) -> List[List[str]]:
        """Build sample row list for gisad csv file"""

        sample_rows = []
        for sample in gisaid_samples:
            sample_row: List[str] = self.get_sample_row(gisaid_sample=sample)
            sample_rows.append(sample_row)
        return sample_rows

    def build_gisaid_csv(self, gisaid_samples: List[GisaidSample], file_name: str) -> Path:
        """Build batch upload_results_to_gisaid csv."""

        file: Path = Path(file_name)

        with open(file_name, "w", newline="\n") as gisaid_csv:
            wr = csv.writer(gisaid_csv, delimiter=",")
            wr.writerow(HEADERS)
            sample_rows: List[List[str]] = self.get_sample_rows(gisaid_samples=gisaid_samples)
            wr.writerows(sample_rows)

        return file

    def get_gisaid_log_file_path(self, case_id: str) -> Path:
        """Path for gisaid bundle log"""

        log_file = Path(self.gisaid_log_dir, case_id).with_suffix(".log")
        if not log_file.parent.exists():
            raise ValueError(f"Gisaid log dir: {self.gisaid_log_dir} doesnt exist")
        if not log_file.exists():
            log_file.touch()
        return log_file

    def append_log(self, temp_log: Path, gisaid_log: Path) -> None:
        """appends temp log to gisaid log and delete temp file"""

        with open(str(temp_log.absolute()), "r") as open_temp_log:
            new_log_data: List = json.load(open_temp_log)
            if gisaid_log.stat().st_size != 0:
                with open(str(gisaid_log.absolute()), "r") as open_gisaid_log:
                    old_log_data: List = json.load(open_gisaid_log)
                    new_log_data.extend(old_log_data)

        with open(str(gisaid_log.absolute()), "w") as open_gisaid_log:
            json.dump(new_log_data, open_gisaid_log)
        temp_log.unlink()

    def upload_results_to_gisaid(self, files: UploadFiles) -> None:
        """Load batch data to GISAID using the gisiad cli."""

        temp_log_file = tempfile.NamedTemporaryFile(
            dir=self.gisaid_log_dir, mode="w+", delete=False
        )
        load_call: list = [
            "--logfile",
            temp_log_file.name,
            "CoV",
            "upload",
            "--csv",
            str(files.csv_file.absolute()),
            "--fasta",
            str(files.fasta_file.absolute()),
        ]
        self.process.run_command(parameters=load_call)
        self.append_log(temp_log=Path(temp_log_file.name), gisaid_log=files.log_file)
        temp_log_file.close()
        if self.process.stderr:
            LOG.info(f"gisaid stderr:\n{self.process.stderr}")
        if self.process.stdout:
            LOG.info(f"gisaid stdout:\n{self.process.stdout}")

    def __str__(self):
        return f"GisaidAPI(dry_run: {self.dry_run})"

    def get_accession_numbers(self, log_file: Path) -> Dict[str, str]:
        """parse accession numbers and sample ids from log file"""

        LOG.info("Parsing accesion numbers from log file")
        accession_numbers = {}
        if log_file.stat().st_size != 0:
            with open(str(log_file.absolute())) as log_file:
                log_data = json.load(log_file)
                for log in log_data:
                    if log.get("code") != "epi_isl_id":
                        continue
                    accession_obj = GisaidAccession(log_message=log.get("msg"))
                    accession_numbers[accession_obj.sample_id] = accession_obj.accession_nr

        return accession_numbers

    def get_completion_file_data(
        self, completion_file: Path, completion_data: Dict[str, str]
    ) -> List[List[str]]:
        """Get completion file with accession numbers"""

        new_completion_file_data = [["provnummer", "urvalskriterium", "GISAID_accession"]]
        with open(str(completion_file.absolute()), "r") as file:
            completion_file_reader = csv.DictReader(file, delimiter=",")
            for sample in completion_file_reader:
                sample_id = sample["provnummer"]
                selection_criteria = sample["urvalskriterium"]
                completion_nr = completion_data.get(sample_id, "")
                new_completion_file_data.append([sample_id, selection_criteria, completion_nr])
        return new_completion_file_data

    def update_completion_file(
        self, completion_file: Path, new_completion_file_data: List[List[str]]
    ) -> None:
        """Update completion file with accession numbers"""

        with open(str(completion_file.absolute()), "w", newline="\n") as file:
            LOG.info("writing accession numbers to new completion file")
            writer = csv.writer(file)
            writer.writerows(new_completion_file_data)

    def ensure_log_in_housekeeper(self, case_id: str, file_path: Path, tags: List[str]) -> None:
        """Adding file to housekeeper if it doesnt exist"""

        if not self.housekeeper_api.find_file_in_latest_version(case_id=case_id, tags=tags):
            self.housekeeper_api.add_and_include_file_to_latest_version(
                case_id=case_id, file=file_path, tags=tags
            )

    def get_upload_files(self, gisaid_samples: List[GisaidSample], case_id: str) -> UploadFiles:
        """Get files for upload to gisaid"""
        return UploadFiles(
            csv_file=self.build_gisaid_csv(
                gisaid_samples=gisaid_samples, file_name=f"{case_id}.csv"
            ),
            fasta_file=self.build_gisaid_fasta(
                gisaid_samples=gisaid_samples, file_name=f"{case_id}.fasta", case_id=case_id
            ),
            log_file=self.get_gisaid_log_file_path(case_id=case_id),
        )

    def manage_completion_file_data(self, case_id: str, accession_numbers: Dict[str, str]) -> None:
        """Merging old completion file data with new"""
        completion_file: Optional[File] = self.housekeeper_api.find_file_in_latest_version(
            case_id=case_id, tags=["komplettering"]
        )
        if not completion_file:
            msg = f"completion file missing for bundle {case_id}"
            raise HousekeeperFileMissingError(message=msg)
        new_completion_data: List[List[str]] = self.get_completion_file_data(
            completion_file=completion_file, completion_data=accession_numbers
        )
        self.update_completion_file(
            completion_file=completion_file,
            new_completion_file_data=new_completion_data,
        )

    def unlik_tmp_files(self, files: UploadFiles) -> None:
        """Unlinking tmp files"""
        files.csv_file.unlink()
        files.fasta_file.unlink()

    def upload(self, case_id: str) -> None:
        """Uploading results to gisaid and saving the accession numbers in completion file"""

        gisaid_samples: List[GisaidSample] = self.get_gisaid_samples(case_id=case_id)
        files: UploadFiles = self.get_upload_files(gisaid_samples, case_id)
        accession_numbers: Dict[str, str] = self.get_accession_numbers(log_file=files.log_file)
        if len(accession_numbers) == len(gisaid_samples):
            self.unlik_tmp_files(files=files)
            raise GisaidUploadFailedError(
                message=f"The samples in bundle {case_id} are already uploaded to GISAID and can not be uploaded more "
                f"than once. "
            )

        try:
            self.upload_results_to_gisaid(files=files)
            self.unlik_tmp_files(files=files)
        except Exception as e:
            raise GisaidUploadFailedError(message=str(e))

        self.ensure_log_in_housekeeper(
            case_id=case_id, tags=["gisaid-log"], file_path=files.log_file
        )

        accession_numbers: Dict[str, str] = self.get_accession_numbers(log_file=files.log_file)
        if accession_numbers:
            self.manage_completion_file_data(case_id=case_id, accession_numbers=accession_numbers)

        if len(accession_numbers) != len(gisaid_samples):
            raise AccessionNumerMissingError(
                message=f"Not all samples in the bundle {case_id} have been uploaded to gisaid."
            )



from smtplib import SMTP

from cg.exc import EmailNotSentError
from cg.models.email import EmailInfo
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart


def send_mail(email_info: EmailInfo) -> None:
    """Email handler. Sending new user request email to admin"""

    with SMTP(email_info.smtp_server) as server:
        msg = MIMEMultipart()
        msg["From"] = email_info.sender_email
        msg["To"] = email_info.receiver_email
        msg["Subject"] = email_info.subject
        msg.attach(MIMEText(email_info.message, "plain"))
        try:
            server.connect()
            server.sendmail(
                from_addr=email_info.sender_email,
                to_addrs=email_info.receiver_email,
                msg=msg.as_string(),
            )
        except:
            raise EmailNotSentError(message=f"Email could not be sent")



from pathlib import Path
from typing import Optional

from cg.apps.orderform.excel_orderform_parser import ExcelOrderformParser
from cg.constants import Pipeline
from cg.meta.orders import OrderType
from cg.models.orders.excel_sample import ExcelSample
from cg.models.orders.orderform_schema import Orderform


def get_sample_obj(
    order_form_parser: ExcelOrderformParser, sample_id: str
) -> Optional[ExcelSample]:
    for sample_obj in order_form_parser.samples:
        if sample_obj.name == sample_id:
            return sample_obj


def is_excel(file_path: Path) -> bool:
    return Path(file_path).suffix == ".xlsx"


def test_parse_mip_rna_orderform(mip_rna_orderform: str):
    """Test to parse an mip rna orderform in excel format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(mip_rna_orderform))
    # GIVEN a orderform API
    orderform_parser: ExcelOrderformParser = ExcelOrderformParser()

    # WHEN parsing the orderform
    orderform_parser.parse_orderform(excel_path=mip_rna_orderform)

    # THEN assert that the project type is correct
    assert orderform_parser.project_type == OrderType.MIP_RNA


def test_parse_balsamic_orderform(balsamic_orderform: str):
    """Test to parse an balsamic orderform in excel format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(balsamic_orderform))
    # GIVEN a orderform API
    orderform_parser: ExcelOrderformParser = ExcelOrderformParser()

    # WHEN parsing the orderform
    orderform_parser.parse_orderform(excel_path=balsamic_orderform)

    # THEN assert that the project type is correct
    assert orderform_parser.project_type == OrderType.BALSAMIC


def test_parse_microbial_orderform(microbial_orderform: str):
    """Test to parse a microbial orderform in excel format"""
    # GIVEN a order form in excel format
    assert is_excel(Path(microbial_orderform))
    # GIVEN a orderform API
    orderform_parser: ExcelOrderformParser = ExcelOrderformParser()

    # WHEN parsing the orderform
    orderform_parser.parse_orderform(excel_path=microbial_orderform)

    # THEN assert that the project type is correct
    assert orderform_parser.project_type == OrderType.MICROSALT


def test_parse_sarscov2_orderform(sarscov2_orderform: str):
    """Test to parse a sarscov2 orderform in excel format"""

    # GIVEN a order form in excel format
    assert is_excel(Path(sarscov2_orderform))
    # GIVEN a orderform API
    orderform_parser: ExcelOrderformParser = ExcelOrderformParser()

    # WHEN parsing the orderform
    orderform_parser.parse_orderform(excel_path=sarscov2_orderform)

    # THEN assert that the project type is correct
    assert orderform_parser.project_type == OrderType.SARS_COV_2


def test_parse_metagenome_orderform(metagenome_orderform: str):
    """Test to parse an metagenome orderform in excel format"""
    # GIVEN a order form in excel format
    assert is_excel(Path(metagenome_orderform))
    # GIVEN a orderform API
    orderform_parser: ExcelOrderformParser = ExcelOrderformParser()

    # WHEN parsing the orderform
    orderform_parser.parse_orderform(excel_path=metagenome_orderform)

    # THEN assert that the project type is correct
    assert orderform_parser.project_type == OrderType.METAGENOME


def test_parse_external_orderform(external_orderform: str):
    """Test to parse a external orderform in xlsx format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(external_orderform))
    # GIVEN a orderform API
    order_form_parser = ExcelOrderformParser()

    # WHEN parsing the mip orderform
    order_form_parser.parse_orderform(excel_path=external_orderform)

    # THEN assert that the project type is correct
    assert order_form_parser.project_type == OrderType.EXTERNAL


def test_generate_external_orderform_with_case(external_order_parser: ExcelOrderformParser):
    """Test to generate a orderform based on parsing of an external orderform"""
    # GIVEN a external orderform parser

    # WHEN generating the orderform
    orderform: Orderform = external_order_parser.generate_orderform()

    # THEN assert that the customer id was correct
    assert external_order_parser.customer_id == "cust002"
    # THEN asert that there where cases in the order
    assert len(orderform.cases) > 0

    case_obj = orderform.cases[0]
    # THEN assert that the case name is as expected
    assert case_obj.name == "fam2"
    assert case_obj.priority == "standard"
    assert set(case_obj.panels) == {"CILM", "CTD"}


def test_generate_mip_orderform_with_cases(mip_order_parser: ExcelOrderformParser):
    """Test to parse a mip orderform with cases"""
    # GIVEN a mip orderform parser

    # WHEN generating a orderform
    orderform: Orderform = mip_order_parser.generate_orderform()

    # THEN assert that there where cases in the order
    assert len(orderform.cases) > 0

    case_obj = orderform.cases[0]
    assert len(case_obj.samples) == 3
    assert case_obj.name == "mipdnacase1"
    assert case_obj.priority == "research"
    assert set(case_obj.panels) == set(["AD-HSP", "Ataxi", "Actionable"])


def test_parse_mip_orderform(mip_orderform: str, nr_samples_mip_orderform: int):
    """Test to parse a mip orderform in xlsx format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(mip_orderform))
    # GIVEN a orderform API
    order_form_parser = ExcelOrderformParser()
    # GIVEN the correct orderform name
    order_name: str = Path(mip_orderform).stem

    # WHEN parsing the mip orderform
    order_form_parser.parse_orderform(excel_path=mip_orderform)

    # THEN assert the number of samples parsed are correct
    assert len(order_form_parser.samples) == nr_samples_mip_orderform

    # THEN assert that the project type is correct
    assert order_form_parser.project_type == str(Pipeline.MIP_DNA)


def test_parse_rml_orderform(rml_orderform: str, nr_samples_rml_orderform: int):
    """Test to parse an excel orderform in xlsx format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(rml_orderform))
    # GIVEN a orderform API
    order_form_parser = ExcelOrderformParser()
    # GIVEN the correct orderform name
    order_name: str = Path(rml_orderform).stem

    # WHEN parsing the RML orderform
    order_form_parser.parse_orderform(excel_path=rml_orderform)

    # THEN assert that the correct name was set
    assert order_form_parser.order_name == order_name

    # THEN assert that the number of samples was correct
    assert len(order_form_parser.samples) == nr_samples_rml_orderform


def test_parse_fastq_orderform(fastq_orderform: str, nr_samples_fastq_orderform: int):
    """Test to parse an fastq orderform in xlsx format"""
    # GIVEN a orderform in excel format
    assert is_excel(Path(fastq_orderform))
    # GIVEN a orderform API
    order_form_parser = ExcelOrderformParser()
    # GIVEN the correct orderform name
    order_name: str = Path(fastq_orderform).stem

    # WHEN parsing the fastq orderform
    order_form_parser.parse_orderform(excel_path=fastq_orderform)

    # THEN assert that the correct name was set
    assert order_form_parser.order_name == order_name

    # THEN assert that the correct number if samples where parsed
    assert len(order_form_parser.samples) == nr_samples_fastq_orderform

    # THEN it should determine the project type
    assert order_form_parser.project_type == str(Pipeline.FASTQ)

    # THEN it should determine the correct customer should have been parsed
    assert order_form_parser.customer_id == "cust000"


def test_fastq_samples_is_correct(fastq_order_parser: ExcelOrderformParser):
    """Test that everything was correctly parsed from the fastq order"""
    # GIVEN a orderform parser where a fastq order is parsed

    # GIVEN a tumor and normal sample with known information
    tumor_sample_id = "fastqsample1"
    normal_sample_id = "fastqsample2"

    # WHEN fetching the tumor and the normal sample
    tumour_sample = get_sample_obj(fastq_order_parser, tumor_sample_id)
    normal_sample = get_sample_obj(fastq_order_parser, normal_sample_id)

    # THEN assert that they where both parsed
    assert tumour_sample and normal_sample


def test_generate_parsed_rml_orderform(rml_order_parser: ExcelOrderformParser, caplog):
    """Test to generate a order from a parsed rml excel file"""
    # GIVEN a order form parser that have parsed an excel file

    # WHEN generating the order
    order: Orderform = rml_order_parser.generate_orderform()

    # THEN assert that some samples where parsed and found
    assert order.samples
    # THEN assert that no cases where found since this is an RML order
    assert not order.cases



from typing import Optional

from sqlalchemy import Column, ForeignKey, UniqueConstraint, orm, types
from sqlalchemy.orm.exc import NoResultFound

from .base import Model


class Project(Model):

    project_id = Column(types.Integer, primary_key=True)
    projectname = Column(types.String(255), nullable=False)
    comment = Column(types.Text)
    time = Column(types.DateTime, nullable=False)

    samples = orm.relationship("Sample", cascade="all", backref=orm.backref("project"))

    def __repr__(self):
        return u"{self.__class__.__name__}: {self.project_id}".format(self=self)

    @staticmethod
    def exists(project_name: str) -> Optional[int]:
        """Checks if the Project entry already exists"""
        try:
            project: Project = Project.query.filter_by(projectname=project_name).one()
            return project.project_id
        except NoResultFound:
            return None



import logging
import yaml

from pathlib import Path
from typing import Any, List, Optional

from cg.apps.mip import parse_trending
from cg.apps.mip.confighandler import ConfigHandler
from cg.constants import COLLABORATORS, COMBOS, MASTER_LIST, Pipeline
from cg.constants.tags import HkMipAnalysisTag
from cg.exc import CgError
from cg.meta.workflow.analysis import AnalysisAPI
from cg.meta.workflow.fastq import MipFastqHandler
from cg.models.cg_config import CGConfig
from cg.models.mip.mip_sample_info import MipBaseSampleInfo, parse_sample_info
from cg.store import models

CLI_OPTIONS = {
    "config": {"option": "--config_file"},
    "priority": {"option": "--slurm_quality_of_service"},
    "email": {"option": "--email"},
    "base": {"option": "--cluster_constant_path"},
    "dryrun": {"option": "--dry_run_all"},
    "gene_list": {"option": "--vcfparser_slt_fl"},
    "max_gaussian": {"option": "--gatk_varrecal_snv_max_gau"},
    "skip_evaluation": {"option": "--qccollect_skip_evaluation"},
    "start_with": {"option": "--start_with_recipe"},
}

LOG = logging.getLogger(__name__)


class MipAnalysisAPI(AnalysisAPI):
    """The workflow is accessed through Trailblazer but cg provides additional conventions and
    hooks into the status database that makes managing analyses simpler"""

    def __init__(self, config: CGConfig, pipeline: Pipeline):
        super().__init__(pipeline, config)

    @property
    def root(self) -> str:
        raise NotImplementedError

    @property
    def conda_env(self) -> str:
        raise NotImplementedError

    @property
    def mip_pipeline(self) -> str:
        raise NotImplementedError

    @property
    def script(self) -> str:
        raise NotImplementedError

    @property
    def fastq_handler(self):
        return MipFastqHandler

    def get_pedigree_config_path(self, case_id: str) -> Path:
        return Path(self.root, case_id, "pedigree.yaml")

    def get_case_config_path(self, case_id: str) -> Path:
        return Path(self.root, case_id, "analysis", f"{case_id}_config.yaml")

    def get_deliverables_file_path(self, case_id: str) -> Path:
        """
        Location in working directory where deliverables file will be stored upon completion of analysis.
        Deliverables file is used to communicate paths and tag definitions for files in a finished analysis
        """
        return Path(self.root, case_id, "analysis", f"{case_id}_deliverables.yaml")

    def get_sample_info_path(self, case_id: str) -> Path:
        """Get case analysis sample info path"""
        return Path(self.root, case_id, "analysis", f"{case_id}_qc_sample_info.yaml")

    def resolve_panel_bed(self, panel_bed: Optional[str]) -> Optional[str]:
        if panel_bed:
            if panel_bed.endswith(".bed"):
                return panel_bed
            bed_version = self.status_db.bed_version(panel_bed)
            if not bed_version:
                raise CgError("Please provide a valid panel shortname or a path to panel.bed file!")
            return bed_version.filename

    def pedigree_config(self, case_id: str, panel_bed: str = None) -> dict:
        """Make the MIP pedigree config. Meta data for the family is taken from the family object
        and converted to MIP format via trailblazer.
        """

        # Validate and reformat to MIP pedigree config format
        case_obj: models.Family = self.status_db.family(case_id)
        return ConfigHandler.make_pedigree_config(
            data={
                "case": case_obj.internal_id,
                "default_gene_panels": case_obj.panels,
                "samples": [
                    self.config_sample(link_obj=link_obj, panel_bed=panel_bed)
                    for link_obj in case_obj.links
                ],
            }
        )

    def write_pedigree_config(self, case_id: str, data: dict) -> None:
        """Write the pedigree config to the the case dir"""
        out_dir = self.get_case_path(case_id=case_id)
        out_dir.mkdir(parents=True, exist_ok=True)
        pedigree_config_path = self.get_pedigree_config_path(case_id=case_id)
        pedigree_config_path.write_text(yaml.dump(data))
        LOG.info("Config file saved to %s", pedigree_config_path)

    @staticmethod
    def get_sample_data(link_obj: models.FamilySample) -> dict:
        return {
            "sample_id": link_obj.sample.internal_id,
            "sample_display_name": link_obj.sample.name,
            "analysis_type": link_obj.sample.application_version.application.analysis_type,
            "sex": link_obj.sample.sex,
            "phenotype": link_obj.status,
            "expected_coverage": link_obj.sample.application_version.application.min_sequencing_depth,
        }

    def get_sample_fastq_destination_dir(
        self, case_obj: models.Family, sample_obj: models.Sample
    ) -> Path:
        return Path(
            self.root,
            case_obj.internal_id,
            sample_obj.application_version.application.analysis_type,
            sample_obj.internal_id,
            "fastq",
        )

    def link_fastq_files(self, case_id: str, dry_run: bool = False) -> None:
        case_obj = self.status_db.family(case_id)
        for link in case_obj.links:
            self.link_fastq_files_for_sample(
                case_obj=case_obj,
                sample_obj=link.sample,
            )

    def write_panel(self, case_id: str, content: List[str]):
        """Write the gene panel to case dir"""
        out_dir = Path(self.root, case_id)
        out_dir.mkdir(parents=True, exist_ok=True)
        out_path = Path(out_dir, "gene_panels.bed")
        with out_path.open("w") as out_handle:
            out_handle.write("\n".join(content))

    @staticmethod
    def convert_panels(customer: str, default_panels: List[str]) -> List[str]:
        """Convert between default panels and all panels included in gene list."""
        # check if all default panels are part of master list
        if customer in COLLABORATORS and all(panel in MASTER_LIST for panel in default_panels):
            return MASTER_LIST

        # the rest are handled the same way
        all_panels = set(default_panels)

        # fill in extra panels if selection is part of a combo
        for panel in default_panels:
            if panel in COMBOS:
                for extra_panel in COMBOS[panel]:
                    all_panels.add(extra_panel)

        # add OMIM to every panel choice
        all_panels.add("OMIM-AUTO")

        return list(all_panels)

    def _get_latest_raw_file(self, family_id: str, tag: str) -> Any:
        """Get a python object file for a tag and a family ."""

        last_version = self.housekeeper_api.last_version(bundle=family_id)

        analysis_files = self.housekeeper_api.files(
            bundle=family_id, version=last_version.id, tags=[tag]
        ).all()

        if analysis_files:
            analysis_file_raw = self._open_bundle_file(analysis_files[0].path)
        else:
            raise LOG.warning(
                "No post analysis files received from DeliverAPI for '%s'",
                family_id,
            )

        return analysis_file_raw

    def _open_bundle_file(self, relative_file_path: str) -> Any:
        """Open a bundle file and return it as an Python object."""

        full_file_path = Path(self.housekeeper_api.get_root_dir()).joinpath(relative_file_path)
        return yaml.safe_load(open(full_file_path))

    def get_latest_metadata(self, family_id: str) -> dict:
        """Get the latest trending data for a family."""

        mip_config_raw = self._get_latest_raw_file(family_id=family_id, tag=HkMipAnalysisTag.CONFIG)
        qcmetrics_raw = self._get_latest_raw_file(
            family_id=family_id, tag=HkMipAnalysisTag.QC_METRICS
        )
        sampleinfo_raw = self._get_latest_raw_file(
            family_id=family_id, tag=HkMipAnalysisTag.SAMPLE_INFO
        )
        trending = dict()
        if mip_config_raw and qcmetrics_raw and sampleinfo_raw:
            try:
                trending = parse_trending.parse_mip_analysis(
                    mip_config_raw=mip_config_raw,
                    qcmetrics_raw=qcmetrics_raw,
                    sampleinfo_raw=sampleinfo_raw,
                )
            except KeyError as error:
                LOG.warning(
                    "get_latest_metadata failed for '%s', missing key: %s",
                    family_id,
                    error,
                )
                LOG.warning(
                    "get_latest_metadata failed for '%s', missing key: %s",
                    family_id,
                    error,
                )
                trending = {}
        return trending

    @staticmethod
    def is_dna_only_case(case_obj: models.Family) -> bool:
        """Returns True if all samples of a case has dna application type"""

        return all(
            _link.sample.application_version.application.analysis_type not in "wts"
            for _link in case_obj.links
        )

    def get_skip_evaluation_flag(self, case_id: str, skip_evaluation: bool) -> bool:
        """If any sample in this case is downsampled or external, returns true"""
        if skip_evaluation:
            return True
        case_obj = self.status_db.family(case_id)
        for link_obj in case_obj.links:
            downsampled = isinstance(link_obj.sample.downsampled_to, int)
            external = link_obj.sample.application_version.application.is_external
            if downsampled or external:
                LOG.info(
                    "%s: downsampled/external - skip evaluation",
                    link_obj.sample.internal_id,
                )
                return True
        return False

    def get_cases_to_analyze(self) -> List[models.Family]:
        cases_query: List[models.Family] = self.status_db.cases_to_analyze(
            pipeline=self.pipeline, threshold=self.threshold_reads
        )
        cases_to_analyze = []
        for case_obj in cases_query:
            if case_obj.action == "analyze" or not case_obj.latest_analyzed:
                cases_to_analyze.append(case_obj)
            elif (
                self.trailblazer_api.get_latest_analysis_status(case_id=case_obj.internal_id)
                == "failed"
            ):
                cases_to_analyze.append(case_obj)
        return cases_to_analyze

    @staticmethod
    def _append_value_for_non_flags(parameters: list, value) -> None:
        """Add the value of the non boolean options to the parameters"""
        if value is not True:
            parameters.append(value)

    @staticmethod
    def _cg_to_mip_option_map(parameters: list, mip_key) -> None:
        """Map cg options to MIP option syntax"""
        parameters.append(CLI_OPTIONS[mip_key]["option"])

    def run_analysis(self, case_id: str, command_args: dict, dry_run: bool) -> None:
        parameters = [
            case_id,
        ]
        for key, value in command_args.items():
            if value:
                self._cg_to_mip_option_map(parameters, key)
                self._append_value_for_non_flags(parameters, value)

        exit_code = self.process.run_command(dry_run=dry_run, parameters=parameters)
        for line in self.process.stdout_lines():
            LOG.info(line)
        for line in self.process.stderr_lines():
            LOG.info(line)
        return exit_code

    def get_case_path(self, case_id: str) -> Path:
        return Path(self.root, case_id)

    def get_trailblazer_config_path(self, case_id: str) -> Path:
        return Path(self.get_case_path(case_id=case_id), "analysis", "slurm_job_ids.yaml")

    def config_sample(self, link_obj: models.FamilySample, panel_bed: str) -> dict:
        raise NotImplementedError

    def get_pipeline_version(self, case_id: str) -> str:
        """Get MIP version from sample info file"""
        LOG.debug("Fetch pipeline version")
        sample_info_raw = yaml.safe_load(self.get_sample_info_path(case_id).open())
        sample_info: MipBaseSampleInfo = parse_sample_info(sample_info_raw)
        return sample_info.mip_version

    def get_cases_to_store(self) -> List[models.Family]:
        """Retrieve a list of cases where analysis finished successfully,
        and is ready to be stored in Housekeeper"""
        finished_cases: List[models.Family] = [
            case_object
            for case_object in self.get_running_cases()
            if self.trailblazer_api.is_latest_analysis_completed(case_id=case_object.internal_id)
        ]

        return finished_cases



"""Tests for the config part of Crunchy"""
import json
from pathlib import Path

from cg.apps.crunchy.files import get_crunchy_metadata, update_metadata_date
from cgmodels.crunchy.metadata import CrunchyMetadata


def test_get_spring_metadata_real_file(real_spring_metadata_path: Path, crunchy_config_dict: dict):
    """Test to parse the content of a real spring metadata file"""
    # GIVEN the path to a file with spring metadata content
    with open(real_spring_metadata_path, "r") as infile:
        content = json.load(infile)

    # WHEN parsing the content
    crunchy_metadata: CrunchyMetadata = get_crunchy_metadata(real_spring_metadata_path)

    # THEN assert the content is the same
    assert len(content) == len(crunchy_metadata.files)


def test_update_date(spring_metadata_file: Path, crunchy_config_dict: dict):
    """Test to update the date in a spring metadata file"""
    # GIVEN the path to a metadata file without any "updated" information and a crunchy api
    spring_metadata: CrunchyMetadata = get_crunchy_metadata(spring_metadata_file)
    for file_info in spring_metadata.files:
        assert file_info.updated is None

    # WHEN running the update date function
    update_metadata_date(spring_metadata_file)

    # THEN assert that the "updated" information has been added
    updated_spring_metadata: CrunchyMetadata = get_crunchy_metadata(spring_metadata_file)
    for file_info in updated_spring_metadata.files:
        assert file_info.updated is not None
